### Multinomial Logit model estimation

### Create the data array from the existing dataset
Data <- read.csv("data.csv",header=TRUE)
## Count the number of data rows
hh <- nrow(Data)

## Set the initial values of the parameters (the number in parenthesis corresponds to the number of parameters to be estimated)
b0 <- numeric(5)

## Recall that we estimate parameters by maximizing the log-likelihood function, hence we first need to define that function

##### Define the log-likelihood function of the logit model#####

fr <- function(x) {
  ### declare the parameters###
  #  b1 <- x[1] # tachiyori
  #  b2 <- x[2] # arrival time at company (minutes from 0 o'clock)
  #  b3 <- x[3] # commutation minutes
  #  b4 <- x[4] # commute method
  #  b5 <- x[5] # wednesday dummy # monday dummy
  #  b6 <- x[6] # friday dummy
  #  b7 <- x[7] # gender (male dummy)
  #  b8 <- x[8] # landprice around company
  #  b9 <- x[9] # landprice around home
  b1 <- x[1] # tachiyori
  b2 <- x[2] # departure time from company (minutes from 0 o'clock)
  b3 <- x[3] # commute method (train dummy)
  b4 <- x[4] # friday dummy
  b5 <- x[5] # tachiyori last week
  #  b1 <- x[1] # tachiyori
  #  b2 <- x[2] # arrival time at company (minutes from 0 o'clock)
  #  b3 <- x[3] # working time (minutes)
  #  b4 <- x[4] # commute method (car dummy)
  #  b5 <- x[5] # friday dummy
  #  b6 <- x[6] # tachiyori last week

  ## declare the log-likelihood variable, set value to 0
  LL = 0
  
  ### For this choice problem, we consider the following 5 modes???
  ## tachiyori
  ## not tachiyori
  
  ## calculate the utility function: :introduce the desired explanatory variables in the function
  tyokki <- exp(0)
  #  shiji <- exp(b1*matrix(1,nrow =hh,ncol=1) + b2*Data$arrivalTimeAtCompany/1000 
  #              + b3*Data$commutationMinutes/100 + b4*(Data$commutationMethod=="")
  #              + b5*(Data$wday=="j") + b6*(Data$wday=="j") + b7*(Data$genderID==1) 
  #              + b8*Data$landpriceAroundCompany/1000 + b9*Data$landpriceAroundHome/1000)
  shiji <- exp(b1*matrix(1,nrow =hh,ncol=1) 
              + b2*Data$departureTimeFromCompany/1000
              + b3*(Data$commutationMethod=="S")
              + b4*(Data$wday=="j") + b5*(Data$tachiyoriLastWeek))
  #  shiji <- exp(b1*matrix(1,nrow =hh,ncol=1) 
  #              + b2*Data$arrivalTimeAtCompany/1000
  #              + b3*Data$commutationMinutes/100
  #              + b4*(Data$commutationMethod=="")
  #              + b5*(Data$wday=="j")+ b6*(Data$tachiyoriLastWeek))

  ### Calculate the choice probabilities
  ## calculate the Inclusive Value (the denominator of the choice probabilities equation)
  deno <- (tyokki + shiji)

  ## Calculate indiviudal choice probabilities
  Ptyokki <- tyokki / deno
  Pshiji  <- shiji / deno
  
  ## Avoid problems stemming from choice probabilities becoming zero.選択確??????0になってしまっぁE??合に起こる問顁E??回避
  Ptyokki <- (Ptyokki!=0)*Ptyokki + (Ptyokki==0)
  Pshiji  <- (Pshiji!=0)*Pshiji + (Pshiji==0)

  ## Choice results
  Ctyokki <- (!Data$tachiyori)
  Cshiji  <- (Data$tachiyori)

  ## Calculate the Log-likelihood function
  LL <- colSums(Ctyokki*log(Ptyokki) + Cshiji*log(Pshiji))

  return(LL)
}

##### Maximize the Log-likelihood function#####

##Parameter optimization 
res <- optim(b0,fr,gr=NULL, method = "Nelder-Mead", hessian = TRUE, control=list(fnscale=-1,maxit=5000))

## Parmeter estimation、Hessian matrix calculation 
b   <- res$par
hhh <- res$hessian

## Calculate the t-statistic
tval <- b/sqrt(-diag(solve(hhh)))

## L(0), Log-Likelihood when all parameters are 0
L0 <- fr(b0)
## LL, maximium likelihood
LL <- res$value

##### Output #####
print(res)
## L(0)
print(L0)
## LL
print(LL)
##rho-square
print((L0-LL)/L0)
## adjusted rho-square
print((L0-(LL-length(b)))/L0)
##estimated parameter values
print(b)
## t-statistic 
print(tval)

##### Simulation #####
## ̎Lm
tyokki <- exp(0)
shiji  <- exp(res$par[1]*matrix(1,nrow =hh,ncol=1) 
            + res$par[2]*Data$departureTimeFromCompany/1000
            + res$par[3]*(Data$commutationMethod=="S")
            + res$par[4]*(Data$wday=="j") + res$par[5]*(Data$tachiyoriLastWeek))
denom <- tyokki + shiji
prob <- shiji/denom
mean(prob)
## SSpꍇ̎Lm
tyokki <- exp(0)
shiji  <- exp(res$par[1]*matrix(1,nrow =hh,ncol=1) 
            + res$par[2]*Data$departureTimeFromCompany/1000
            + res$par[3]*matrix(1,nrow =hh,ncol=1) # SS𗘗pꍇ
            + res$par[4]*(Data$wday=="j") + res$par[5]*(Data$tachiyoriLastWeek))
denom <- tyokki + shiji
prob <- shiji/denom
mean(prob)
## ދΎԂ1ԑ܂ꍇ̎Lm
tyokki <- exp(0)
shiji  <- exp(res$par[1]*matrix(1,nrow =hh,ncol=1) 
            + res$par[2]*(Data$departureTimeFromCompany - 60)/1000
            + res$par[3]*(Data$commutationMethod=="S")
            + res$par[4]*(Data$wday=="j") + res$par[5]*(Data$tachiyoriLastWeek))
denom <- tyokki + shiji
prob <- shiji/denom
mean(prob)
## ދΎԂ2ԑ܂ꍇ̎Lm
tyokki <- exp(0)
shiji  <- exp(res$par[1]*matrix(1,nrow =hh,ncol=1) 
            + res$par[2]*(Data$departureTimeFromCompany - 120)/1000
            + res$par[3]*(Data$commutationMethod=="S")
            + res$par[4]*(Data$wday=="j") + res$par[5]*(Data$tachiyoriLastWeek))
denom <- tyokki + shiji
prob <- shiji/denom
mean(prob)