### Nested Logit model estimation

### Create the data array from the existing dataset
Data <- read.csv("data.csv",header=T)
## Count the number of data rows
hh <- nrow(Data)

## Set the initial values of the parameters (the number in parenthesis corresponds to the number of parameters to be estimated)
b0 <- numeric(9) 

##### Define the log-likelihood function of the logit model#####

fr <- function(x) {
  b1 <- 0
  b2 <- 0
  b3 <- 0
  b4 <- 0
  b5 <- 0
  b6 <- 0
  b7 <- 0
  b8 <- 0
  b9 <- 0
  b10 <- 0
  b11 <- 0
  b12 <- 0
  b13 <- 0
  b14 <- 0
  b15 <- 0
  b16 <- 0
  bmonday_t <- 0
  btuesday_t <- 0
  bavg_t <- 0
  bmonday <- 0
  btuesday <- 0
  bavg <- 0

  ### declare the parameters###
  b1 <- x[1] # tachiyori
  b2 <- x[2] # arrival time at company (minutes from 0 o'clock)
#  b3 <- x[3] # commutation minutes
  b4 <- x[3] # commute method
  b5 <- x[4] # wednesday dummy
  b6 <- x[5] # friday dummy
#  b7 <- x[7] # gender(male dummy)
#  b8 <- x[7] # landprice around company
#  b9 <- x[8] # landprice around home
  b10 <- x[6] # late flag
#  b11 <- x[8] # [for late] arrival time at company (minutes from 0 o'clock)
#  b12 <- x[9] # [for late] commutation minutes
#  b13 <- x[10] # [for late] commute method
#  b14 <- x[4] # [for late] wednesday dummy
#  b15 <- x[5] # [for late] friday dummy
#  b16 <- x[13] # [for late] gender(male dummy)

#  bmonday_t <- x[11]
#  btuesday_t <- x[12]
#  bavg_t <- x[9]
  bmonday <- x[7]
  btuesday <- x[8]
#  bavg <- x[9]

  ##Scale parameter
  pa <- x[9] 

  # threshold value between early and old
  basetime <- 1140 # 19:00 = 1140 min. from 0:00

  ## Choice results
#  C.late   <- (Data$departureTimeFromCompany >= basetime)
  C.late   <- (Data$departureTimeFromCompany >= basetime) & (!as.logical(Data$tachiyori))
  C.tyokki <- (Data$departureTimeFromCompany < basetime) & (!as.logical(Data$tachiyori))
#  C.shiji  <- (Data$departureTimeFromCompany < basetime) & (as.logical(Data$tachiyori))
  C.shiji  <- (as.logical(Data$tachiyori))


  ## declare the log-likelihood variable, set value to 0
  LL = 0

  ### For this choice problem, we consider the following 5 modes
  ## tyokki(early)
  ## shiji (early)
  ##       (late)

  ## calculate the utility function: :introduce the desired explanatory variables in the function
  tyokki <- exp(0)

  shiji <- exp(b1*matrix(1,nrow =hh,ncol=1)
              + b2*Data$arrivalTimeAtCompany/1000
              + b3*Data$commutationMinutes/100 + b4*(Data$commutationMethod=="")
              + b5*(Data$wday=="j") + b6*(Data$wday=="j")
               + b7*(Data$genderID==1) 
              + b8*Data$landpriceAroundCompany/1000 + b9*Data$landpriceAroundHome/1000
              + bmonday_t*(Data$wday=="j") + btuesday_t*(Data$wday=="Ηj")
              + bavg_t*(Data$averageDepartureTimeFromCompany<basetime)
             )

  late <- exp(b10*matrix(1,nrow =hh,ncol=1) + b11*Data$arrivalTimeAtCompany/1000
              + b12*Data$commutationMinutes/100 + b13*(Data$commutationMethod=="")
              + b14*(Data$wday=="j") + b15*(Data$wday=="j") + b16*(Data$genderID==1)
              + bmonday*(Data$wday=="j") + btuesday*(Data$wday=="Ηj")
              + bavg*(Data$averageDepartureTimeFromCompany<basetime)
              ) 

  ### Construct nests.First, calculate public or private choice probabilityP(LV1)
  ## setting of log-sum variables
#  logsum.early  <- log( ( (tyokki+shiji)!=0) * (tyokki+shiji) + ((tyokki+shiji)==0))
#  logsum.late <- log((late!=0) * late + (late==0))
  logsum.kitaku  <- log( ( (tyokki+late)!=0) * (tyokki+late) + ((tyokki+late)==0))
  logsum.shiji <- log((shiji!=0) * shiji + (shiji==0))

  ##Calculate public or private choice probability with these log-sum variables
#  nume.early <- (logsum.early != 0)*exp(pa*logsum.early) + (logsum.early == 0)
#  nume.late  <- (logsum.late != 0)*exp(pa*logsum.late) + (logsum.late == 0)
  nume.kitaku <- (logsum.kitaku != 0)*exp(pa*logsum.kitaku) + (logsum.kitaku == 0)
  nume.shiji  <- (logsum.shiji != 0)*exp(pa*logsum.shiji) + (logsum.shiji == 0)

#  deno <- nume.early + nume.late
#  P.early <- nume.early  / deno
#  P.late  <- nume.late / deno
   deno <- nume.kitaku + nume.shiji
   P.kitaku <- nume.kitaku / deno
   P.shiji <- nume.shiji / deno

  ### Next, calculate conditional probability.P(LV2|LV1)
  ## conditional probability under public choice
#  deno.early     <- tyokki + shiji
#  P.tyokki.early  <- tyokki / ((deno.early!=0)*deno.early + (deno.early==0))
#  P.shiji.early    <- shiji   / ((deno.early!=0)*deno.early + (deno.early==0))
  deno.kitaku     <- tyokki + late
  P.tyokki.kitaku  <- tyokki / ((deno.kitaku!=0)*deno.kitaku + (deno.kitaku==0))
  P.late.kitaku    <- late   / ((deno.kitaku!=0)*deno.kitaku + (deno.kitaku==0))

  ## conditional probability under private choice
#  deno.late   <- late
#  P.late.late <- late / ((deno.late!=0)*deno.late + (deno.late==0))
  deno.shiji   <- shiji
  P.shiji.shiji <- shiji / ((deno.shiji!=0)*deno.shiji + (deno.shiji==0))

  ### Finally, calculate joint probability. P(LV1, LV2) = P(LV2|LV1)*P(LV1)
#  P.tyokki <- P.tyokki.early * P.early
#  P.shiji  <- P.shiji.early  * P.early
#  P.late   <- P.late.late    * P.late
  P.tyokki <- P.tyokki.kitaku * P.kitaku
  P.late  <- P.late.kitaku  * P.kitaku
  P.shiji   <- P.shiji.shiji    * P.shiji


  ## Avoid problems stemming from choice probabilities becoming zero.
  P.tyokki <- (P.tyokki!=0)*P.tyokki + (P.tyokki==0)
  P.shiji  <- (P.shiji!=0) *P.shiji   + (P.shiji==0)
  P.late   <- (P.late!=0)  *P.late   + (P.late==0)

  ## Calculate the Log-likelihood function
  LL <- colSums(C.tyokki*log(P.tyokki) + C.shiji*log(P.shiji) + C.late *log(P.late))

  return(LL)
}

tmp <- fr(b0)

##### Maximize the Log-likelihood function#####

##Parameter optimization 
res <- optim(b0,fr, method = "Nelder-Mead", hessian = TRUE, control=list(fnscale=-1,maxit=5000))

## Parmeter estimation、Hessian matrix calculation 
b   <- res$par
hhh <- res$hessian

## Calculate the t-statistic
tval <- b/sqrt(-diag(solve(hhh)))

## L(0), Log-Likelihood when all parameters are 0
L0 <- fr(b0)
## LL, maximium likelihood
LL <- res$value

##### Output #####
print(res)
## L(0)
print(L0)
## LL
print(LL)
##rho-square
print((L0-LL)/L0)
## adjusted rho-square
print((L0-(LL-length(b)))/L0)
##estimated parameter values
print(b)
## t-statistic 
print(tval)

